
from google.appengine.api import urlfetch
from google.appengine.ext import webapp
from google.appengine.ext import db
import re, datetime, logging

from misc.BeautifulSoup import BeautifulSoup
from models.result import Result

link_regex = re.compile(r'\s*<rdf:li rdf:resource="([^"]*)">')
phone_regex = re.compile(r'(?<!Id:)(?:[:#\s])(?:(?:\+?1\s*(?:[.-]\s*)?)?(?:\(\s*([2-9]1[02-9]|[2-9][02-8]1|[2-9][02-8][02-9])\s*\)|([2-9]1[02-9]|[2-9][02-8]1|[2-9][02-8][02-9]))\s*(?:[.-]\s*)?)?([2-9]1[02-9]|[2-9][02-9]1|[2-9][02-9]{2})\s*(?:[.-]\s*)?([0-9]{4})(?:\s*(?:#|x\.?|ext\.?|extension)\s*(\d+))?')

rent_regex = re.compile(r'\$(\d{3,4})\s')
bedroom_regex = re.compile(r'\s(\d)br\s')

class PhoneNumber(object):
  def __init__(self, area, zone, num, ext):
    self.area = area
    self.zone = zone
    self.num  = num
    self.ext  = ext

  def __str__(self):
    l = []
    if self.area:
      l.append("(%s)" % self.area)
    l.append("%s-%s" % (self.zone, self.num))
    if self.ext:
      l.append("x%s" % self.ext)
    return " ".join(l)

class Address(object):
  def __init__(self, street, inter_street, city, state):
    self.street = street
    self.inter_street = inter_street
    self.city = city
    self.state = state

  def __str__(self):
    l = []
    if self.inter_street:
      l.append("%s & %s" % (self.street, self.inter_street))
    else:
      l.append(self.street)
    l.append(self.city)
    l.append(self.state)
    return ", ".join(l)

class Retrieve(webapp.RequestHandler):
  def get(self):

    # first delete old postings
    # -------------------------
    results = Result.all().order("posted").fetch(100)
    ten_days_ago = datetime.date.today() - datetime.timedelta(days=10)
    db.delete([r for r in results if r.posted < ten_days_ago])

    # now collect new ones
    # --------------------
    url = "http://madison.craigslist.org/apa/index.rss"
    result = urlfetch.fetch(url)
    assert result.status_code == 200, (url, result.status_code)

    start = result.content.find('<rdf:Seq>')
    end   = result.content.find('</rdf:Seq>', start)
    list  = result.content[start:end]

    rpcs = []
    for line in list.splitlines():
      match = link_regex.match(line)
      if match:
        url = match.group(1)
        rpc = urlfetch.create_rpc()
        urlfetch.make_fetch_call(rpc, url)
        rpcs.append((rpc, url, line))

    for rpc, url, line in rpcs:
      result = rpc.get_result()
      if result.status_code == 404:
        logging.debug("status 404 for %s" % url)
        continue

      assert result.status_code == 200, result.status_code

      prologue_string = '<div id="userbody">'
      prologue, body = result.content.split(prologue_string, 1)

      epilogue_string = '<!-- START CLTAGS -->'
      body, epilogue = body.split(epilogue_string, 1)

      soup = BeautifulSoup(body)
      if soup.script:
        soup.script.clear()
      desc = ''.join(soup.findAll(text=True))
      has_html = "<table" in body or "<div" in body

      # get rent
      # --------
      rent = None

      match = rent_regex.search(line)
      if match:
        rent = int(match.group(1))

      # get bedrooms (if any)
      # ---------------------
      size = None

      match = bedroom_regex.search(line)
      if match:
        size = int(match.group(1))
      else:
        if "studio" in line.lower() or "efficiency" in line.lower():
          size = 0

      # search for phone number
      # -----------------------
      phone = None

      match = phone_regex.search(desc)
      if match:
        area = match.group(1)
        if match.group(2):
          area = match.group(2)
        
        phone = str(PhoneNumber(
            area, match.group(3), match.group(4), match.group(5)))

      # compute address & location
      # --------------------------
      address = None
      location = None

      cltags = {}
      tags = epilogue.split("<!-- CLTAG")
      for tag in tags[1:]:
        tag = tag.split("-->", 1)[0]
        key, value = tag.split("=")
        cltags[key.strip()] = value.strip()

      if cltags.get("xstreet0"):
        address = unicode(str(Address(
            cltags["xstreet0"], 
            cltags["xstreet1"], 
            cltags["city"], 
            cltags["region"])), errors='ignore')
      
      location = cltags.get("GeographicArea")
      if location:
        location = unicode(location, errors='ignore')

      # extract map
      # -----------

      gmap = epilogue.split('">google map</a>', 1)
      if len(gmap) > 1:
        assert len(gmap) == 2, len(gmap)
        gmap = unicode(gmap[0].split('href="')[-1],
            errors='ignore')
      else:
        gmap = None

      # extract date
      # ------------

      posted = prologue.split("Date:")[-1].split(",")[0].strip()
      year, month, day = [int(x) for x in posted.split("-")]
      posted = datetime.date(year, month, day)

      # extract posting id and put in db
      # --------------------------------

      posting_id = url.split("/")[-1].split(".")[0]
      Result(
          key_name = posting_id,
          posted   = posted,
          size     = size,
          rent     = rent,
          desc     = desc,
          phone    = phone,
          address  = address,
          location = location,
          src_url  = url,
          gmap_url = gmap,
          has_html = has_html).put()


